package de.w665.fluidcms.db;

import com.rethinkdb.RethinkDB;
import com.rethinkdb.gen.exc.ReqlOpFailedError;
import com.rethinkdb.net.Connection;
import io.quarkus.runtime.Startup;
import jakarta.inject.Inject;
import jakarta.inject.Singleton;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Singleton
public class RethinkDBService {

    @Inject
    RethinkDBConfig config;

    @Inject
    RethinkDBConnector connector;

    private Connection connection;
    private RethinkDB r;

    @Startup
    public void init() {
        this.r = connector.getR();
        this.connection = connector.getConnection();
        initialize();
    }

    public void initialize() {

        log.debug("Initializing RethinkDB database...");
        // rethinkdb check if database exists
        try {
            r.dbCreate(config.getDatabase()).run(connection).stream();
            log.debug("Database " + config.getDatabase() + " created");
        } catch (ReqlOpFailedError e) {
            log.debug("Database " + config.getDatabase() + " already exists. Error: " + e.getClass().getSimpleName());
        }
    }
}
