package de.w665.fluidcms.db.repo;

import com.rethinkdb.RethinkDB;
import com.rethinkdb.net.Connection;
import de.w665.fluidcms.db.RethinkDBConfig;
import de.w665.fluidcms.db.RethinkDBConnector;
import de.w665.fluidcms.db.entity.CObjectEntity;
import de.w665.fluidcms.model.CObject;
import jakarta.annotation.PostConstruct;
import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@ApplicationScoped
public class CObjectRepository {

    @Inject
    RethinkDBConnector connector;

    @Inject
    RethinkDBConfig config;

    @Inject
    ObjectTypeRepository objectTypeRepository;

    private final String TABLE_NAME = "c_objects";
    private RethinkDB r;
    private Connection connection;

    @PostConstruct
    public void init() {
        this.r = this.connector.getR();
        this.connection = this.connector.getConnection();

        // Check if table exists
        try {
            r.db(config.getDatabase()).tableCreate(TABLE_NAME).run(connection);
            log.debug("Table " + TABLE_NAME + " created");
        } catch (Exception e) {
            log.debug("Table " + TABLE_NAME + " already exists");
            if (config.isAutoresetonstartup()) {
                r.db(config.getDatabase()).tableDrop(TABLE_NAME).run(connection);
                r.db(config.getDatabase()).tableCreate(TABLE_NAME).run(connection);
                log.debug("Table " + TABLE_NAME + " reset");
            }
        }
    }

    public void insert(CObject cObject) {
        // Create entity CObject and replace ObjectType with a reference to the object type (e.g. ObjectType ID)
        CObjectEntity entity = new CObjectEntity(cObject.getId(), cObject.getName(), cObject.getType().getId(), cObject.getAttributes());
        r.db(config.getDatabase()).table(TABLE_NAME).insert(entity).run(connection);
    }

    public void insert(CObjectEntity entity) {
        r.db(config.getDatabase()).table(TABLE_NAME).insert(entity).run(connection);
    }

    public CObject get(String id) {
        CObjectEntity entity = r.db(config.getDatabase()).table(TABLE_NAME).filter(r.hashMap("id", id)).run(connection, CObjectEntity.class).first();
        if(entity == null) {
            log.warn("Entity CObject with ID {} not found", id);
            return null;
        }
        // Construct CObject from entity
        return new CObject(entity.getId(), entity.getName(), objectTypeRepository.get(entity.getId()), entity.getAttributes());
    }
}
