package de.w665.fluidcms.db.repo;

import com.rethinkdb.RethinkDB;
import com.rethinkdb.net.Connection;
import de.w665.fluidcms.db.RethinkDBConfig;
import de.w665.fluidcms.db.RethinkDBConnector;
import de.w665.fluidcms.model.ObjectType;
import jakarta.annotation.PostConstruct;
import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@ApplicationScoped
public class ObjectTypeRepository {

    @Inject
    RethinkDBConnector connector;

    @Inject
    RethinkDBConfig config;

    private final String TABLE_NAME = "object_types";
    private RethinkDB r;
    private Connection connection;

    @PostConstruct
    public void init() {
        this.r = this.connector.getR();
        this.connection = this.connector.getConnection();

        // Check if table exists
        try {
            r.db(config.getDatabase()).tableCreate(TABLE_NAME).run(connection);
            log.debug("Table " + TABLE_NAME + " created");
        } catch (Exception e) {
            log.debug("Table " + TABLE_NAME + " already exists");
            if(config.isAutoresetonstartup()) {
                r.db(config.getDatabase()).tableDrop(TABLE_NAME).run(connection);
                r.db(config.getDatabase()).tableCreate(TABLE_NAME).run(connection);
                log.debug("Table " + TABLE_NAME + " reset");
            }
        }
    }

    public void insert(ObjectType objectType) {
        objectType.setId(r.uuid().run(connection, String.class).first());
        r.db(config.getDatabase()).table(TABLE_NAME).insert(objectType).run(connection);
    }

    public ObjectType get(String id) {
        return r.db(config.getDatabase()).table(TABLE_NAME).filter(r.hashMap("id", id)).run(connection, ObjectType.class).first();
    }
}
