package de.w665.fluidcms.rest.mapping;

import de.w665.fluidcms.db.entity.CObjectEntity;
import de.w665.fluidcms.db.repo.CObjectRepository;
import de.w665.fluidcms.db.repo.ObjectTypeRepository;
import de.w665.fluidcms.model.ObjectAttribute;
import de.w665.fluidcms.model.ObjectType;
import jakarta.inject.Inject;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.Produces;
import lombok.extern.slf4j.Slf4j;
import org.jboss.resteasy.reactive.RestResponse;

import java.util.Map;

@Slf4j
@Path("/object")
public class ObjectResource {

    @Inject
    ObjectTypeRepository objectTypeRepository;

    @Inject
    CObjectRepository cObjectRepository;

    @POST
    @Consumes("application/json")
    @Produces("application/json")
    public RestResponse<CObjectEntity> createObject(CObjectEntity objectEntity) {
        // Verify if objectType exists
        // Verify if attributes are valid (matching the object type)
        // Insert object into database
        log.debug(objectEntity.toString());
        ObjectType objectType = objectTypeRepository.get(objectEntity.getObjectTypeId());
        if(objectType == null) {
            log.error("Object type with ID {} not found", objectEntity.getObjectTypeId());
            return RestResponse.status(400, "Invalid object type.");
        }
        // Loop through attributes and verify if they match the object type attributes
        for(String key : objectEntity.getAttributes().keySet()) {
            if(objectType.getAttributes().stream().noneMatch(attr -> attr.getName().equals(key))) {
                log.error("Attribute {} not found in object type {}", key, objectType.getName());
                return RestResponse.status(400, "Invalid attribute.");
            }
        }

        // Verify (type) if object attributes are match object type attributes
        for (Map.Entry<String, ObjectAttribute> entry : objectEntity.getAttributes().entrySet()) {
            String key = entry.getKey();
            ObjectAttribute entityAttribute = entry.getValue();

            // Find the corresponding attribute in the objectType
            ObjectAttribute objectTypeAttribute = objectType.getAttributes().stream()
                    .filter(attr -> attr.getName().equals(key))
                    .findFirst()
                    .orElse(null);

            if (objectTypeAttribute == null) {
                log.error("Attribute {} not found in object type {}", key, objectType.getName());
                return RestResponse.status(400, "Invalid attribute.");
            }

            // Check if the value type is valid
            if (!objectTypeAttribute.verifyValueType(entityAttribute.getValue())) {
                log.error("Attribute {} type does not match object type {}", key, objectType.getName());
                return RestResponse.status(400, "Invalid attribute type.");
            }
        }

        cObjectRepository.insert(objectEntity);

        return RestResponse.ok(objectEntity);
    }
}
