package de.w665.fluidcms;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.w665.fluidcms.model.CObject;
import de.w665.fluidcms.model.ObjectAttribute;
import de.w665.fluidcms.model.ObjectType;
import de.w665.fluidcms.model.objectattributes.StringObjectAttribute;
import io.restassured.http.ContentType;
import org.junit.jupiter.api.Test;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import static io.restassured.RestAssured.given;

public class ObjectResourceTest {
    @Test
    void testObjectEndpoint() throws JsonProcessingException {
        ObjectMapper objectMapper = new ObjectMapper();

        // Create sample data using Java objects
        StringObjectAttribute attribute1 = new StringObjectAttribute();
        attribute1.setName("Attribute 13");
        attribute1.setDescription("This is attribute 13");
        attribute1.setRequired(true);
        attribute1.setDefaultValue("Default Value 13");
        attribute1.setValue("Attribute 13 Value");

        ObjectType objectType = new ObjectType();
        objectType.setName("Object Type 71");
        objectType.setDescription("This is object type 71");
        objectType.setIcon("icon.png");
        objectType.setColor("blue");
        objectType.setAttributes(Arrays.asList(attribute1));

        Map<String, ObjectAttribute> attributes = new HashMap<>();
        attributes.put(attribute1.getName(), attribute1);

        CObject sampleData = new CObject();
        sampleData.setName("Object 71");
        sampleData.setType(objectType);
        sampleData.setAttributes(attributes);

        // Serialize Java object to JSON
        String jsonSampleData = objectMapper.writeValueAsString(sampleData);

        // Perform the request and validate the response
        String responseJson = given()
                .contentType(ContentType.JSON)
                .body(jsonSampleData)
                .when().post("/api/v1/object")
                .then()
                .statusCode(200)
                .extract().asString();

        // Deserialize response JSON to Java object
        CObject responseData = objectMapper.readValue(responseJson, CObject.class);

        // Perform assertions on the deserialized Java object
        assert responseData.getId() != null;
        assert responseData.getName().equals("Object 71");
        assert responseData.getType().getName().equals("Object Type 71");
        assert responseData.getAttributes().size() == 1;

        StringObjectAttribute responseAttribute1 = (StringObjectAttribute) responseData.getAttributes().get("Attribute 13");
        assert responseAttribute1.getType().equals("StringObjectAttribute");
        assert responseAttribute1.getName().equals("Attribute 13");
        assert responseAttribute1.getDescription().equals("This is attribute 13");
        assert responseAttribute1.isRequired();
        assert responseAttribute1.getDefaultValue().equals("Default Value 13");
        assert responseAttribute1.getValue().equals("Attribute 13 Value");
    }
}
